//
// Copyright 2019 Pixar
//
// Licensed under the terms set forth in the LICENSE.txt file available at
// https://openusd.org/license.
//
#ifndef PXR_USD_IMAGING_USD_SKEL_IMAGING_PACKAGE_H
#define PXR_USD_IMAGING_USD_SKEL_IMAGING_PACKAGE_H

#include "pxr/pxr.h"
#include "pxr/usdImaging/usdSkelImaging/api.h"
#include "pxr/base/tf/token.h"

PXR_NAMESPACE_OPEN_SCOPE

USDSKELIMAGING_API
TfToken UsdSkelImagingPackageSkinningShader();

PXR_NAMESPACE_CLOSE_SCOPE

#endif // PXR_USD_IMAGING_USD_SKEL_IMAGING_PACKAGE_H
