//
// Copyright 2016 Pixar
//
// Licensed under the terms set forth in the LICENSE.txt file available at
// https://openusd.org/license.
//
#ifndef PXR_USD_IMAGING_USD_RI_PXR_IMAGING_VERSION_H
#define PXR_USD_IMAGING_USD_RI_PXR_IMAGING_VERSION_H

#include "pxr/pxr.h"

PXR_NAMESPACE_OPEN_SCOPE

//        1: Clone off PXR_USD_IMAGING_USD_RI_PXR_IMAGING_VERSION_H and add light filter support
//  1 ->  2: Add support for camera projection plugins.

#define USD_RI_PXR_IMAGING_API_VERSION 2

PXR_NAMESPACE_CLOSE_SCOPE

#endif // PXR_USD_IMAGING_USD_RI_PXR_IMAGING_VERSION_H
